from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.auto_role import AutoRole
from bot_graphql.abc.data_query_abc import DataQueryABC
from bot_graphql.filter.server_filter import ServerFilter


class AutoRoleQuery(DataQueryABC):
    def __init__(
        self,
        auto_role_rules: AutoRoleRepositoryABC,
        servers: ServerRepositoryABC,
    ):
        DataQueryABC.__init__(self, "AutoRole")

        self._auto_role_rules = auto_role_rules
        self._servers = servers

        self.set_field("id", self.resolve_id)
        self.set_field("channel_id", self.resolve_channel_id)
        self.set_field("message_id", self.resolve_message_id)
        self.set_field("server", self.resolve_server)
        self._add_collection(
            "auto_role_rule", lambda x, *_: self._auto_role_rules.get_auto_role_rules_by_auto_role_id(x.auto_role_id)
        )

    @staticmethod
    def resolve_id(x: AutoRole, *_):
        return x.auto_role_id

    @staticmethod
    def resolve_channel_id(x: AutoRole, *_):
        return x.discord_channel_id

    @staticmethod
    def resolve_message_id(x: AutoRole, *_):
        return x.discord_message_id

    def resolve_server(self, x: AutoRole, *_, filter: ServerFilter = None):
        if filter is not None:
            return filter.filter(self._servers.get_server_by_id(x.server_id))

        return self._servers.get_server_by_id(x.server_id)
